/*******************************************************************************
* File Name: GPRS_1_INT.c
* Version 2.30
*
* Description:
*  This file provides all Interrupt Service functionality of the UART component
*
* Note:
*  Any unusual or non-standard behavior should be noted here. Other-
*  wise, this section should remain blank.
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "GPRS_1.h"
#include "CyLib.h"


/***************************************
* Custom Declratations
***************************************/
/* `#START CUSTOM_DECLARATIONS` Place your declaration here */

/* `#END` */

#if( (GPRS_1_RX_ENABLED || GPRS_1_HD_ENABLED) && \
     (GPRS_1_RXBUFFERSIZE > GPRS_1_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: GPRS_1_RXISR
    ********************************************************************************
    *
    * Summary:
    *  Interrupt Service Routine for RX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  GPRS_1_rxBuffer - RAM buffer pointer for save received data.
    *  GPRS_1_rxBufferWrite - cyclic index for write to rxBuffer,
    *     increments after each byte saved to buffer.
    *  GPRS_1_rxBufferRead - cyclic index for read from rxBuffer,
    *     checked to detect overflow condition.
    *  GPRS_1_rxBufferOverflow - software overflow flag. Set to one
    *     when GPRS_1_rxBufferWrite index overtakes
    *     GPRS_1_rxBufferRead index.
    *  GPRS_1_rxBufferLoopDetect - additional variable to detect overflow.
    *     Set to one when GPRS_1_rxBufferWrite is equal to
    *    GPRS_1_rxBufferRead
    *  GPRS_1_rxAddressMode - this variable contains the Address mode,
    *     selected in customizer or set by UART_SetRxAddressMode() API.
    *  GPRS_1_rxAddressDetected - set to 1 when correct address received,
    *     and analysed to store following addressed data bytes to the buffer.
    *     When not correct address received, set to 0 to skip following data bytes.
    *
    *******************************************************************************/
    CY_ISR(GPRS_1_RXISR)
    {
        uint8 readData;
        uint8 increment_pointer = 0u;
        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START GPRS_1_RXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        readData = GPRS_1_RXSTATUS_REG;

        if((readData & (GPRS_1_RX_STS_BREAK | GPRS_1_RX_STS_PAR_ERROR |
                        GPRS_1_RX_STS_STOP_ERROR | GPRS_1_RX_STS_OVERRUN)) != 0u)
        {
            /* ERROR handling. */
            /* `#START GPRS_1_RXISR_ERROR` */

            /* `#END` */
        }

        while((readData & GPRS_1_RX_STS_FIFO_NOTEMPTY) != 0u)
        {

            #if (GPRS_1_RXHW_ADDRESS_ENABLED)
                if(GPRS_1_rxAddressMode == (uint8)GPRS_1__B_UART__AM_SW_DETECT_TO_BUFFER)
                {
                    if((readData & GPRS_1_RX_STS_MRKSPC) != 0u)
                    {
                        if ((readData & GPRS_1_RX_STS_ADDR_MATCH) != 0u)
                        {
                            GPRS_1_rxAddressDetected = 1u;
                        }
                        else
                        {
                            GPRS_1_rxAddressDetected = 0u;
                        }
                    }

                    readData = GPRS_1_RXDATA_REG;
                    if(GPRS_1_rxAddressDetected != 0u)
                    {   /* store only addressed data */
                        GPRS_1_rxBuffer[GPRS_1_rxBufferWrite] = readData;
                        increment_pointer = 1u;
                    }
                }
                else /* without software addressing */
                {
                    GPRS_1_rxBuffer[GPRS_1_rxBufferWrite] = GPRS_1_RXDATA_REG;
                    increment_pointer = 1u;
                }
            #else  /* without addressing */
                GPRS_1_rxBuffer[GPRS_1_rxBufferWrite] = GPRS_1_RXDATA_REG;
                increment_pointer = 1u;
            #endif /* End SW_DETECT_TO_BUFFER */

            /* do not increment buffer pointer when skip not adderessed data */
            if( increment_pointer != 0u )
            {
                if(GPRS_1_rxBufferLoopDetect != 0u)
                {   /* Set Software Buffer status Overflow */
                    GPRS_1_rxBufferOverflow = 1u;
                }
                /* Set next pointer. */
                GPRS_1_rxBufferWrite++;

                /* Check pointer for a loop condition */
                if(GPRS_1_rxBufferWrite >= GPRS_1_RXBUFFERSIZE)
                {
                    GPRS_1_rxBufferWrite = 0u;
                }
                /* Detect pre-overload condition and set flag */
                if(GPRS_1_rxBufferWrite == GPRS_1_rxBufferRead)
                {
                    GPRS_1_rxBufferLoopDetect = 1u;
                    /* When Hardware Flow Control selected */
                    #if(GPRS_1_FLOW_CONTROL != 0u)
                    /* Disable RX interrupt mask, it will be enabled when user read data from the buffer using APIs */
                        GPRS_1_RXSTATUS_MASK_REG  &= (uint8)~GPRS_1_RX_STS_FIFO_NOTEMPTY;
                        CyIntClearPending(GPRS_1_RX_VECT_NUM);
                        break; /* Break the reading of the FIFO loop, leave the data there for generating RTS signal */
                    #endif /* End GPRS_1_FLOW_CONTROL != 0 */
                }
            }

            /* Check again if there is data. */
            readData = GPRS_1_RXSTATUS_REG;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START GPRS_1_RXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End GPRS_1_RX_ENABLED && (GPRS_1_RXBUFFERSIZE > GPRS_1_FIFO_LENGTH) */


#if(GPRS_1_TX_ENABLED && (GPRS_1_TXBUFFERSIZE > GPRS_1_FIFO_LENGTH))


    /*******************************************************************************
    * Function Name: GPRS_1_TXISR
    ********************************************************************************
    *
    * Summary:
    * Interrupt Service Routine for the TX portion of the UART
    *
    * Parameters:
    *  None.
    *
    * Return:
    *  None.
    *
    * Global Variables:
    *  GPRS_1_txBuffer - RAM buffer pointer for transmit data from.
    *  GPRS_1_txBufferRead - cyclic index for read and transmit data
    *     from txBuffer, increments after each transmited byte.
    *  GPRS_1_rxBufferWrite - cyclic index for write to txBuffer,
    *     checked to detect available for transmission bytes.
    *
    *******************************************************************************/
    CY_ISR(GPRS_1_TXISR)
    {

        #if(CY_PSOC3)
            uint8 int_en;
        #endif /* CY_PSOC3 */

        /* User code required at start of ISR */
        /* `#START GPRS_1_TXISR_START` */

        /* `#END` */

        #if(CY_PSOC3)   /* Make sure nested interrupt is enabled */
            int_en = EA;
            CyGlobalIntEnable;
        #endif /* CY_PSOC3 */

        while((GPRS_1_txBufferRead != GPRS_1_txBufferWrite) &&
             ((GPRS_1_TXSTATUS_REG & GPRS_1_TX_STS_FIFO_FULL) == 0u))
        {
            /* Check pointer. */
            if(GPRS_1_txBufferRead >= GPRS_1_TXBUFFERSIZE)
            {
                GPRS_1_txBufferRead = 0u;
            }

            GPRS_1_TXDATA_REG = GPRS_1_txBuffer[GPRS_1_txBufferRead];

            /* Set next pointer. */
            GPRS_1_txBufferRead++;
        }

        /* User code required at end of ISR (Optional) */
        /* `#START GPRS_1_TXISR_END` */

        /* `#END` */

        #if(CY_PSOC3)
            EA = int_en;
        #endif /* CY_PSOC3 */

    }

#endif /* End GPRS_1_TX_ENABLED && (GPRS_1_TXBUFFERSIZE > GPRS_1_FIFO_LENGTH) */


/* [] END OF FILE */
